#include <stdio.h>

#include "imic.h"
#include "imconn.h"
#include "impn.h"
#include "packet.h"
#include "keybind.h"
#include "anthyconv.h"

IMIC::IMIC(IMConn *conn,int imid,int icid)
{
  mConn = conn;
  mICid = icid;
  mIMid = imid;
  mAC = 0;
  mPrevPeLen = 0;
}

void IMIC::forward_event(RxPacket *p)
{
  int t;
  t = p->getC32();
  switch(t){
  case 0:// STRING
    printf("STRING forwarded\n");
    break;
  case 1:// TEXT
    printf("TEXT forwarded\n");
    break;
  case 2:// KEYEVENT
    int nr = p->getC32();
    proc_key_event(p);
    break;
  }
}

void IMIC::proc_key_event(RxPacket *p)
{
  keyEvent ev;
  ev.keycode = p->getC32();
  ev.keychar = p->getC32();
  ev.modifier = p->getC32();
  ev.timestamp = p->getC32();

  if ( !mAC ){
    mAC = create_anthy_context();
  }
  int mask=0;
  if ( mAC ){
    mask = mAC->proc_key(&ev);
  }
  if ( mask & COMMIT_RAW ){
    TxPacket *t = createTxPacket(IM_COMMIT_STRING);
    t->pushC16(mIMid);
    t->pushC16(mICid);
    t->pushC32(0);// STRING
    STRING str = createSTRING("BAKA");
    t->pushSTRING(str);
    mConn->pushTxPacket(t);
  }
  if ( mask & UPDATE_PREEDIT ){
    TxPacket *t = createTxPacket(IM_PREEDIT_START);
    t->pushC16(mIMid);
    t->pushC16(mICid);
    mConn->pushTxPacket(t);
  }
  if ( mask & COMMIT_PENDING ){
    TxPacket *t = createTxPacket(IM_COMMIT_STRING);
    t->pushC16(mIMid);
    t->pushC16(mICid);
    t->pushC32(0);// STRING
    STRING str = mAC->get_pending();
    t->pushSTRING(str);
    mConn->pushTxPacket(t);
  }
}

void IMIC::preedit_draw(int l)
{
  TxPacket *t = createTxPacket(IM_PREEDIT_DRAW);
  t->pushC16(mIMid);
  t->pushC16(mICid);
  t->pushC32(0);//caret
  t->pushC32(0);//first
  t->pushC32(mPrevPeLen);//prev len
  composePreedit(t);
  mConn->pushTxPacket(t);
}

void IMIC::composePreedit(TxPacket *t)
{
  preedit p;
  mAC->get_preedit(&p);
  int seg_count = p.segs.size();
  int fb_count = 0;
  int char_count = 0;
  int i;

  t->pushC32(1);//TEXT
  for ( i = 0 ; i < seg_count ; i++){
    fb_count += p.segs[i].str.size()*12;
    char_count += p.segs[i].str.size();
  }
  mPrevPeLen = char_count;
  t->pushC32(fb_count);
  for ( i = 0 ; i < seg_count ; i++){
    STRING::iterator j;
    for (j = p.segs[i].str.begin() ; j != p.segs[i].str.end() ; j++){
      t->pushC16(*j);
      t->pushC16(8);
      t->pushC32(0);
      t->pushC32(1);
    }
  }
  t->pushC32(0);// no annotation
}

void IMIC::preedit_done()
{
  TxPacket *t = createTxPacket(IM_PREEDIT_DONE);
  t->pushC16(mIMid);
  t->pushC16(mICid);
  mConn->pushTxPacket(t);
}
